# Author: hevans@nerdkits.com

import select
import socket
import serial
import time
import sys
import threading
import signal
import socket


# serial-comm.py is mean to do serial communications with the NerdKits LED Array
# Input is taken from std in so data can be piped to it from other programms, or just typed on the console
# Provides a socket interface on which programs can check how many messages are currently in the queue.
# The script will respond to any communication over the socket with the number of lines currently waiting to be
# sent.

# sugguested usage
#
# python serial-comm.py
# then type a message and press enter
#
# or
#
# some-other-program-that-outputs-to-stdout | python serial-comm.py
# pipes the output of another program to serial-comm.py, if desired, the other program can quer serial-comm.py
# over the socket to find out how many messages are waiting ot be sent.

serial_comm_dev = "/dev/ttyUSB0"
serial_comm_rate = 115200
query_socket_port = 7777

serial = serial.Serial(serial_comm_dev, serial_comm_rate)
scrollMode = True
buffer = []

class Info(threading.Thread):
  def run(self):
    HOST = ''                 # Symbolic name meaning all available interfaces
    PORT = query_socket_port              # Arbitrary non-privileged port
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind((HOST, PORT))
    while True:
      print "Info: Waitign for Request"
      s.listen(1)
      conn, addr = s.accept()
      print 'Connected by', addr

      data = conn.recv(1024)
      if data:
        print 'Reporting ', str(len(buffer))
        conn.send(str(len(buffer)))
      conn.close()
      

class StdInReader(threading.Thread):
  def run(self):
    global buffer
    while True:
      stdin_data = sys.stdin.readline().upper().strip()
      if stdin_data != '':
        buffer.append(stdin_data + "     ")
      else:
        time.sleep(1)

class LedArraySender(threading.Thread):
  def run(self):
    global buffer
    last_sent = "NO DATA     "
    while True:
      if len(buffer) != 0:
        line = buffer.pop(0)
      else:
        line = last_sent

      print "Sending: '" + line + "' to the the array."
      print "Currently %d lines in buffer" % len(buffer)
      last_sent = line
      for character in line:
        #send one character
        serial.write(character)
        #wait for a response from the NerdKit
        response = serial.read()
         


try:
  producer = StdInReader()
  sender = LedArraySender()
  info = Info()
  producer.start()
  info.start()
  sender.start()
  while True:
    time.sleep(.1)
except KeyboardInterrupt:
  sys.exit()
  
